# pylint: disable=redefined-builtin
# pylint: disable=wildcard-import
from .lists import *
from .dicts import *

__version__ = "0.3.1"

__title__ = "nicetypes"
__description__ = "Advanced types for Python3"
__uri__ = "https://github.com/tvuillemin/nicetypes"
__doc__ = __description__ + " <" + __uri__ + ">"

__author__ = "Thibaut Vuillemin"
__email__ = "contact@thibautvuillemin.com"

__license__ = "MIT"
__copyright__ = "Copyright (c) 2017 Thibaut Vuillemin"



class UniqList:
    """A list-like object that makes sure there is no double in the list.

    When you try to add an element that is already present, UniqList ignores it silently.
    You can view it as a kind of "ordered set", which removes doubles but preserves order.
    An already present element will be prefered over a new one, unless specified otherwise.
    """

    def __init__(self, ordered_iterable=()):
        """Constructor

        _elements_set is a internal set that contains every element in the UniqList.
        This set is used for search operations, because those operations are much
        faster on a set than on a list (complexity O(1) vs O(n)).
        """
        self._elements_list = list()
        self._elements_set = set()
        for element in ordered_iterable:
            self.append(element)

    def __getitem__(self, index):
        return self._elements_list[index]

    def __iter__(self):
        return iter(self._elements_list)

    def __str__(self):
        return "%s(%s)" % (self.__class__.__name__, str(self._elements_list))

    def append(self, element):
        if element not in self._elements_set:
            self._elements_list.append(element)
            self._elements_set.add(element)

    def insert(self, index, element, replace=False):
        if element not in self._elements_set:
            self._elements_list.insert(index, element)
            self._elements_set.add(element)
        elif replace:
            self._elements_list.remove(element)
            self._elements_list.insert(index, element)
        else:  # element already exists but replace is False
            pass

    def extend(self, other):
        for element in other:
            self.append(element)

    def __add__(self, other):
        new_uniq_list = UniqList(self)
        for element in other:
            new_uniq_list.append(element)
        return new_uniq_list



from unittest import TestCase

from nicetypes.dicts import ImmutableDict


class TestImmutableDict(TestCase):

    def test_immutabledict_init(self):
        # Given
        data = {"foo": 1}
        # When
        imd = ImmutableDict(data)
        # Then
        self.assertDictEqual(data, imd._inner_dict)  # pylint: disable=protected-access

    def test_immutabledict_getitem(self):
        # Given
        imd = ImmutableDict({"foo": 1})
        # When
        value = imd["foo"]
        # Then
        self.assertEqual(value, 1)

    def test_immutabledict_str(self):
        # Given
        imd = ImmutableDict({"foo": 1})
        # When
        imd_str = str(imd)
        # Then
        self.assertEqual(imd_str, "ImmutableDict({'foo': 1})")

    def test_immutabledict_len(self):
        # Given
        imd = ImmutableDict({"foo": 1, "bar": 2})
        # When
        imd_len = len(imd)
        # Then
        self.assertEqual(imd_len, 2)

    def test_immutabledict_get(self):
        # Given
        imd = ImmutableDict({"foo": 1})
        # When
        value = imd.get("foo")
        # Then
        self.assertEqual(value, 1)

    def test_immutabledict_items(self):
        # Given
        imd = ImmutableDict({"foo": 1})
        # When
        items = tuple(imd.items())
        # Then
        self.assertSequenceEqual(items, [("foo", 1)])

    def test_immutabledict_keys(self):
        # Given
        imd = ImmutableDict({"foo": 1})
        # When
        keys = tuple(imd.keys())
        # Then
        self.assertSequenceEqual(keys, ["foo"])

    def test_immutabledict_values(self):
        # Given
        imd = ImmutableDict({"foo": 1})
        # When
        values = tuple(imd.values())
        # Then
        self.assertSequenceEqual(values, [1])

    def test_immutabledict_hashable(self):
        # Given
        imd = ImmutableDict({"foo": 1})
        # When
        hash_ = hash(imd)
        # Then
        self.assertIsInstance(hash_, int)
